Attribute VB_Name = "Module1"
Rem Contains procedures etc for Harrassed Booking Clerk
Rem Assume that PersEnq is entity 1
Rem             PhoneEnq is entity 2
Rem             Observer is entity 3
Rem             Clerks are entities 4 onwards

Rem Uses buit-in Rnd for first try

Rem Note that Bs and Cs in the model must be public subs

Rem ----------------------------------------------------------------------
Rem ---------------  DECLARATIONS  ---------------------------------------
Rem ----------------------------------------------------------------------

Option Explicit

Dim PersEnq As Integer              'Personal enquirer arrival machine
Dim PhoneEnq As Integer             'Phone caller arraival machine
Dim Observer As Integer             'Observer entity

Public Obs As Integer                  'Number of observations so far

Public PersIn As Integer               'Number of personal arrivals
Public PersOut As Integer              'Number of completed personal services

Public PhoneIn As Integer              'Number of phone calls made
Public PhoneOut As Integer             'Number of completed phone calls

Public PersQ As Integer                'Size of personal customer queue
Public PhoneQ As Integer               'Size of phone call queue

Public PersRec(200) As Integer         'Observations of PersQ
Public PhoneRec(200) As Integer        'Observations of PhoneQ

Public NumClerks As Integer            'Number of clerks working
Public FirstClerk As Integer           'ID if first clerk entity
Public LastClerk As Integer            'ID of last clerk entity

Public ObsInterval As Integer          'Time interval between observations

Public PersArrTime As Integer          'Mean inter-arrival time, personal enquirers
Public PhoneArrTime As Integer         'Mean inter-arrival time, phone callers
Public PersServeTime As Integer        'Mean personal service time
Public PhoneServeTime As Integer       'Mean phone conversation time

'--------------------------------------------------------------------------------------
'------------------------------- The Bs -----------------------------------------------
'--------------------------------------------------------------------------------------

'Arrival of a personal enquirer
'
Sub B1()

    PersIn = PersIn + 1
    PersQ = PersQ + 1
    If TraceOn Then
        Debug.Print "PersEnq "; Str$(PersIn); " arrives"
    End If
    Schedule PersEnq, 1, -PersArrTime * Log(Rnd(PhoneEnq))
End Sub

Sub B10()

End Sub

'End of a personal service
'
Sub B2()
    
    PersOut = PersOut + 1
    If TraceOn Then
        Debug.Print "End of personal service "; Str$(PersOut); " with clerk "; Str$(GetCurrent - FirstClerk + 1)
    End If
        
End Sub

'Arrival of a phone call
'
Sub B3()

    PhoneIn = PhoneIn + 1
    PhoneQ = PhoneQ + 1
    If TraceOn Then
        Debug.Print "PhoneCall "; Str$(PhoneIn); " arrives"
    End If
    Schedule PhoneEnq, 3, -PhoneArrTime * Log(Rnd(PhoneEnq))

End Sub

'End of a phone conversation
'
Sub B4()
    PhoneOut = PhoneOut + 1
    If TraceOn Then
        Debug.Print "End of phone conversation "; Str$(PhoneOut); " with clerk "; Str$(GetCurrent - FirstClerk + 1)
    End If
    
End Sub

'Observation of queue lengths
Sub B5()

    PersRec(Obs) = PersQ
    PhoneRec(Obs) = PhoneQ
    Obs = Obs + 1

    Schedule Observer, 5, ObsInterval

    If TraceOn Then
        Debug.Print "Observation no: "; Str$(Obs); Str(PersQ)
    End If

End Sub

Sub B6()

End Sub

Sub B7()

End Sub

Sub B8()

End Sub

Sub B9()

End Sub

'--------------------------------------------------------------------------------------
'------------------------------- The Cs -----------------------------------------------
'--------------------------------------------------------------------------------------

'Begin personal service
'
Sub C1()
    
    Dim ThisClerk As Integer

    ThisClerk = FirstClerk - 1

    Do While (PersQ > 0) And (ThisClerk < LastClerk)
        ThisClerk = ThisClerk + 1
        If CheckAvail(ThisClerk) Then
            SetCStarted
            PersQ = PersQ - 1
            Schedule ThisClerk, 2, -PersServeTime * Log(Rnd(ThisClerk))
            If TraceOn Then
                Debug.Print "Start personal service with "; Str$(ThisClerk - FirstClerk + 1)
            End If
        End If
    Loop
    
End Sub

Sub C10()

End Sub

'Begin phone conversation
'
Sub C2()
    Dim ThisClerk As Integer

    ThisClerk = FirstClerk - 1

    Do While (PhoneQ > 0) And (ThisClerk < LastClerk)
        ThisClerk = ThisClerk + 1
        If CheckAvail(ThisClerk) Then
            SetCStarted
            PhoneQ = PhoneQ - 1
            Schedule ThisClerk, 4, -PhoneServeTime * Log(Rnd(ThisClerk))
            If TraceOn Then
                Debug.Print "Start phone conversation with "; Str$(ThisClerk - FirstClerk + 1)
            End If
        End If
    Loop
    

End Sub

Sub C3()

End Sub

Sub C4()

End Sub

Sub C5()

End Sub

Sub C6()

End Sub

Sub C7()

End Sub

Sub C8()

End Sub

Sub C9()

End Sub

'--------------------------------------------------------------------------------------
'------------------------------- Initialisation ---------------------------------------
'--------------------------------------------------------------------------------------

Public Sub Initialisation()
'Sets up simulation ready to run
'Must call ZeroLimits from VBExec
    
    Dim ThisClerk As Integer
    Dim Fred As Single
    
    ZeroLimits
    
    Erase PersRec
    Erase PhoneRec
    Obs = 0
   
    PersEnq = 1
    PhoneEnq = 2
    Observer = 3

    InitEnt 1, -PersArrTime * Log(Rnd(PersEnq)), "Customer"
    InitEnt 3, -PhoneArrTime * Log(Rnd(PhoneEnq)), "Phone call"
    
    InitEnt 5, ObsInterval, "Observer"

    FirstClerk = Observer + 1
    LastClerk = Observer + NumClerks
    
    For ThisClerk = FirstClerk To LastClerk
        InitEntFree "Clerk" + Str$(ThisClerk - FirstClerk + 1)
    Next

End Sub

Private Sub InitValues()
'Initialises simulation counters
    
    PersIn = 0
    PhoneIn = 0
    PersOut = 0
    PhoneOut = 0
    PersQ = 0
    PhoneQ = 0

    NumClerks = 1
    PersArrTime = 10
    PhoneArrTime = 12
    PersServeTime = 8
    PhoneServeTime = 10
    SetRunDuration 100
    ObsInterval = 10
    
    TraceOn = False

End Sub

'--------------------------------------------------------------------------------------
'------------------------------- The Main sub -----------------------------------------
'--------------------------------------------------------------------------------------

Sub Main()

    InitValues
    InitForm.SetPars
    InitForm.Show

End Sub

