Attribute VB_Name = "VBEXEC"
Rem Three phase simulation in Visual Basic 5
Rem Started February 1999
Rem enforces private variables for the excutive
Rem Revised to try use of procedural parameters

Rem -----------------------------------------------------------------------------------------
Rem ---------------------------  Public DECLARATIONS ----------------------------------------
Rem -----------------------------------------------------------------------------------------

Option Explicit

Public Const MaxEnt = 30                            'Max entities in simulation
Public Const MaxC = 10                              'Max Cs in simulation

Type EntDetails                                     'Info for each entity
    Name As String * 20                             'For output
    Avail As Integer                                'Boolean for scheduling
    TimeCell As Integer                             'Next event time if scheduled
    NextAct As Integer                              'Next B if scheduled
    Util As Integer                                 'Total busy time so far
End Type

Private RunDuration As Integer                       'Length of simulation
Private Clock As Integer, PrevClock As Integer       'Current and previous simulation time
Private CurrEnt As Integer                           'Current entity
Private NumCurrEnts As Integer                       'Number of current entities (A & BPhases)
Private NumEnts As Integer                           'Number of entities in simulation
Private CStarted As Integer                          'Boolean check for CPhase
Private NumCs As Integer                             'Number of Cs


Private Details(MaxEnt) As EntDetails                   'Array of entity info
Private CurrEntArray(MaxEnt) As Integer                 'Array of entities found by APhase

Public TraceOn As Integer                           'True if debugging
Public Stopped As Integer                           'True if Interrupted
Public SingleStep As Integer

Rem -----------------------------------------------------------------------------------------
Rem ---------------------------  A, B, & C PHASES -------------------------------------------
Rem -----------------------------------------------------------------------------------------


Public Sub APhase()
'Performs time scan by examining the Details array
'Checks all entities for which Avail is false, looks for smallest time cells in these
'Min time cell entity numbers stored in CurrEntArray

    Dim Entity As Integer                               'Number of entity being inspected
    Dim Minm As Integer                                 'Initial value for minimum search

    NumCurrEnts = 0                                     'Set CurrEnt counter
    Minm = 32767                                        'Set Minm value
    
    For Entity = 1 To NumEnts                           'Go through Details Array
        If Details(Entity).Avail = False Then           'Skip if Avail
            If Details(Entity).TimeCell <= Minm Then    'Find if Minm > TimeCell
                If Details(Entity).TimeCell < Minm Then
                    NumCurrEnts = 1                     'Reset NumCurrEnts
                    Else: NumCurrEnts = NumCurrEnts + 1 'or increment NumCurrEnts
                End If
                Minm = Details(Entity).TimeCell         'Reset Minm
                CurrEntArray(NumCurrEnts) = Entity      'Add Entity number to Array
            End If
        End If
    Next

    If Minm = 32767 Then
        Debug.Print "Error in A Phase, no entities scheduled"
        Stop

    ElseIf Minm < 0 Then
        Debug.Print "Error in A Phase, Minm gone -ve"
        Stop

    Else: Clock = Minm                                  'Found next Clock value

    End If
End Sub


Public Sub BPhase()
'Works through the CurrEnt Array.
'For each entity stored therein it executes the correct B Activity
'Executed after APhase and before CPhase
'Uses clumsy Select Case because of inadequacies of Visual BASIC

    Dim Entity As Integer

    If (TraceOn = True) Then
        Debug.Print "Time now ", Str(Clock)
    End If

    ShowEntDetails                                  'If debugging, show Details array

    For Entity = 1 To NumCurrEnts                   'Work through CurrEntArray
        CurrEnt = CurrEntArray(Entity)              'Take CurrEnt number
        Details(CurrEnt).Avail = True               'Set CurrEnt as Avail
        Select Case Details(CurrEnt).NextAct        'Now do the correct B activity
            Case 1
                B1
            Case 2
                B2
            Case 3
                B3
            Case 4
                B4
            Case 5
                B5
            Case 6
                B6
            Case 7
                B7
            Case 8
                B8
            Case 9
                B9
            Case 10
                B10
            Case Else
                Debug.Print "B number out of range in BPhase"
        End Select
    Next

End Sub


Public Sub CPhase()
'Just tries each C activity in turn until no more activity
'Crude, but that's VBASIC!!
'C's need to be defined by the user

    CStarted = True
    
    Do Until CStarted = False
        CStarted = False
        C1
        C2
        'C3
        'C4
        'C5
        'C6
        'C7
        'C8
        'C9
        'C10
    Loop

End Sub

Public Sub Simulate()
' Calls A, B & C phases after initialisation
' Requires a public Initialisation sub in the model being run
' Would need to be edited if no UpDateScreen() in RunTime form
' There is probably a better way to update the screen via OLE
' Performs no Initialisation - should be done elsewhere

    Do While Clock < RunDuration
        APhase
        BPhase
        CPhase
        RunTime.UpDateScreen
        DoEvents
    Loop

End Sub

Rem -----------------------------------------------------------------------------------------
Rem ---------------------------  Access functions  -------------------------------------------
Rem -----------------------------------------------------------------------------------------

Public Sub SetRunDuration(ByVal RD As Integer)
'Required, since RunDuration is private to this module

    RunDuration = RD
End Sub

Public Function GetRunDuration()
'Access function, required since RunDuration is private to this module

    GetRunDuration = RunDuration
End Function

Public Sub SetClock(ByVal SimClock As Integer)
'Required, since Clock is private to this module

    Clock = SimClock
End Sub

Public Function GetClock()
'Access function, required since Clock is private to this module

    GetClock = Clock
End Function

Public Sub SetPrevClock(ByVal SimPrev As Integer)
'Required, since PrevClock is private to this module

    PrevClock = SimPrev
End Sub

Public Function GetPrevClock()
'Access function, required since PrevClock is private to this module

    GetPrevClock = PrevClock
End Function

Public Function GetCurrent()
'Access function, required since Current is private to this module

    GetCurrent = CurrEnt
End Function

Public Sub SetNumEnts(ByVal NE As Integer)
'Required, since NumEnts is private to this module

    NumEnts = NE
End Sub

Public Function GetNumEnts()
'Access function, required since NumEnts is private to this module

    GetNumEnts = NumEnts
End Function

Public Sub SetCStarted()
'Access function, required since CStarted is private to this module

    CStarted = True
End Sub

Public Sub SetNumCs(ByVal NC As Integer)
'Required, since NumCs is private to this module

    NumCs = NC
End Sub

Public Function CheckAvail(ByVal ThisEnt As Integer) As Boolean
'Returns with True or False, depending on Avail field of ThisEnt

    CheckAvail = Details(ThisEnt).Avail
End Function

Public Function GetUtil(ByVal ThisEnt)
'Returns with the current utilisation of ThisEnt

    GetUtil = Details(ThisEnt).Util
End Function

Rem -----------------------------------------------------------------------------------------
Rem ---------------------------  Useful routines -------------------------------------------
Rem -----------------------------------------------------------------------------------------

Public Sub InitEnt(ByVal ThisAct As Integer, ByVal ThisDuration As Integer, ByVal ThisName As String)
'Creates and schedules an entity
    
    InitEntFree ThisName                          'Use InitEntFree to create it
    Schedule GetNumEnts, ThisAct, ThisDuration       'Now schedule it

End Sub


Public Sub InitEntFree(ByVal ThisName As String)
'Creates the specified entity and sets its state to available
'Assumes that NumEnts is initially set to zero

    SetNumEnts GetNumEnts + 1                  'Increment number of entities
    With Details(GetNumEnts)
        .Name = ThisName        'Give a name
        .TimeCell = 0           'Initialise time cell
        .Util = 0               'Put utilisation to zero
        .Avail = True           'Set state to available
    End With

End Sub

Public Sub Release(ByVal ThisEnt As Integer)
'Releases a Scheduled entity by putting Avail to True

    Details(ThisEnt).Avail = True
    
End Sub

Public Sub Schedule(ByVal ThisEnt As Integer, ByVal NextB As Integer, ByVal ThisDuration As Integer)
'Schedules an entity for a B activity after This Duration has elapsed
'Checks to see that the entity is already free

    With Details(ThisEnt)
        If .Avail = False Then
            Debug.Print "Tried to schedule an already Scheduled entity"
            Stop
        End If

        .Avail = False
        .NextAct = NextB
        If GetClock + ThisDuration > GetRunDuration Then
            ThisDuration = GetRunDuration - GetClock
        End If
        .TimeCell = GetClock + ThisDuration
        .Util = Details(ThisEnt).Util + ThisDuration
    End With
    
End Sub

Public Sub ShowEntDetails()
'Puts Details Array on Debug screen if debugging
    
    Dim ThisEnt As Integer                              'Loop counter
    Dim ThisState As String * 5

    If (TraceOn = True) Then
        Debug.Print "Entity", "State", "TimeCell"       'Show headings

        For ThisEnt = 1 To GetNumEnts
            With Details(ThisEnt)
                If .Avail = True Then                   'Entity state
                    ThisState = "Avail"
                Else: ThisState = "Engaged"
                End If
                Debug.Print .Name; Str$(ThisEnt), ThisState, Str$(.TimeCell)
            End With
        Next
    End If

End Sub

Public Sub ZeroLimits()
'Sets Clock, PrevClock & counters to zero; initialises random number generator
    
    SetClock 0
    SetPrevClock 0
    SetNumEnts 0
    SetNumCs 0

End Sub








